/*
 * lgcontrol.h: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#ifndef __LGCONTROL_H
#define __LGCONTROL_H

#define TIMEOUT                  1.0
#define READ_STATUS              0xFF

#define LGDEFAULTDEVICE          "/dev/ttyUSB0"

#define CONVERTED_2D_3D_DEPTH    14

#define LGMINVOL                 0
#define LGMAXVOL                 64
#define LGVOLSTEP                1

#define QUEUESENDINTERVALL       200

#include "lgcontrol_service.h"

static const char *VERSION        = "0.0.2";
static const char *DESCRIPTION    = "LG-TV remote controled by VDR via rs232";
static const char *MAINMENUENTRY  = NULL;

int AutoPowerOn;
int AutoPowerOff;
int AutoPowerCheck;
int LGVolumeControl;
int LGBaseVolume;
int IncreaseLevel;

// --- LG -----------------------------------------------------------

class LG {
private:
  int fd;
  int baudrate;
  int set_id;
public:
  LG();
  ~LG();
  int SendCommand(char cmd1, char cmd2, int value, int value2=0, int value3=0, int value4=0);
  void InitSerial(const char *device);
  void CloseSerial(void);
  };

// --- CommandListItem ---------------------------------------------------------

class CommandListItem:public cListObject {
private:
  char cmd1, cmd2;
  int value1, value2, value3, value4;
public:
  CommandListItem(char _cmd1, char _cmd2, int _value1, int _value2=0, int _value3=0, int _value4=0) { cmd1=_cmd1; cmd2=_cmd2 ;value1=_value1; value2=_value2; value3=_value3; value4=_value4; }
  char  Cmd1() { return   cmd1; }
  char  Cmd2() { return   cmd2; }
  int Value1() { return value1; }
  int Value2() { return value2; }
  int Value3() { return value3; }
  int Value4() { return value4; }
  };

// --- CommandList -------------------------------------------------------------
class CommandList:public cList<CommandListItem> {
  };

// --- cSendThread -----------------------------------------------------------

class cSendThread:public cThread {
private:
  LG          *lg;
  CommandList *CommandQueue;
  int  SendCommand(char cmd1, char cmd2, int value, int value2=0, int value3=0, int value4=0);
  bool sendinprogress;
  bool extsendinprogress;
protected:
  virtual void Action();
public:
  cSendThread(LG **connection);
  ~cSendThread();
  bool GetSendInProgress() { return sendinprogress; }
  void SetExtSendInProgress(bool progress) { extsendinprogress = progress; }
  const char *Working();
  void AddToCommandQueue(char cmd1, char cmd2, int value, int value2=0, int value3=0, int value4=0);
  bool IsCommandQueueEmpty(){return CommandQueue->First();}
  };

// --- cLGCommands -----------------------------------------------------------

class cLGCommands {
private:
  LG          *lg;
  cSendThread *SendThread;
  int  SendCommandDirectToTV(char Cmd1, char Cmd2, int Value1, int Value2=0, int Value3=0, int Value4=0);
  void SendCommandToCmdQueue(char Cmd1, char Cmd2, int Value1, int Value2=0, int Value3=0, int Value4=0);
public:
  cLGCommands(const char *device);
  ~cLGCommands();

  bool SendRemoteKey(int key);

  void Disable3D()             { SendCommandToCmdQueue('x', 't', 1 ,1, 1); }
  void Enable3DSbS()           { SendCommandToCmdQueue('x', 't', 0, 1, 0); }
  void Enable3DSbSLR()         { SendCommandToCmdQueue('x', 't', 0, 1, 1); }
  void Enable3DTB()            { SendCommandToCmdQueue('x', 't', 0, 0, 1); }
  void Enable3DCB()            { SendCommandToCmdQueue('x', 't', 0, 2, 0); }
  void Enable3DFS()            { SendCommandToCmdQueue('x', 't', 0, 3, 0); }
  void Enable3DConverted2D()   { SendCommandToCmdQueue('x', 't', 3, 0, 0, CONVERTED_2D_3D_DEPTH); }

  void PowerOn()               { SendCommandToCmdQueue('k', 'a', 1); }
  void PowerOff()              { SendCommandToCmdQueue('k', 'a', 0); }
  void TogglePower()           { GetCurrentPowerStatus() ? SendCommandToCmdQueue('k', 'a', 0) : SendCommandToCmdQueue('k', 'a', 1); }
  int  GetCurrentPowerStatus() { return SendCommandDirectToTV('k', 'a', READ_STATUS); }

  void SetVolume(int vol)      { SendCommandToCmdQueue('k', 'f', (vol > LGMAXVOL) ? LGMAXVOL : (vol < LGMINVOL) ? LGMINVOL : vol); }
  int  GetCurrentVolume()      { return SendCommandDirectToTV('k', 'f', READ_STATUS); }

  void ToggleMute()            { IsMute() ? SendCommandToCmdQueue('k', 'e', 01) : SendCommandToCmdQueue('k', 'e', 00); }
  bool IsMute()                { if (SendCommandDirectToTV('k', 'e', READ_STATUS)) { return false; } else { return true;} }

  void ToggleRemoteLocked()    { IsRemoteLocked() ? SendCommandToCmdQueue('k', 'm', 00) : SendCommandToCmdQueue('k', 'm', 01); }
  bool IsRemoteLocked()        { if (SendCommandDirectToTV('k', 'm', READ_STATUS)) { return false; } else { return true;} }
  };

// --- cMenuSetupLGControl -------------------------------------------------

class cMenuSetupLGControl : public cMenuSetupPage {
private:
  int newAutoPowerOff;
  int newAutoPowerOn;
  int newAutoPowerCheck;
  int newLGVolumeControl;
  int newLGBaseVolume;
  int newIncreaseLevel;
protected:
  virtual void Store(void);
public:
  cMenuSetupLGControl(void);
  };

// --- cMyStatusMonitor ------------------------------------------------------

class cMyStatusMonitor : public cStatus {
private:
  cLGCommands *LGCmd;
protected:
  virtual void SetVolume(int Volume, bool Absolute);
public:
  cMyStatusMonitor (cLGCommands **commandqueue);
  };

// --- cPluginLgcontrol ------------------------------------------------------

class cPluginLgcontrol : public cPlugin {
private:
  bool              IsUserInactive;
  const char       *LGDevice;
  cLGCommands      *LGCmd;
  cMyStatusMonitor *statusMonitor;
public:
  cPluginLgcontrol(void);
  virtual ~cPluginLgcontrol();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Stop(void);
  virtual void Housekeeping(void);
  virtual void MainThreadHook(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  virtual cString Active(void);
  virtual time_t WakeupTime(void);
  virtual bool Service(const char *Id, void *Data);
  virtual const char **SVDRPHelpPages(void);
  virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
  };

#endif //__LGCONTROL_H
